from typing import Optional

from pydantic import BaseModel


class StartVerificationResponse(BaseModel):
    """Response object for starting a verification process.

    Args:
        request_id (str): The unique ID of the Verify request. You need this `request_id`
            for the Verify check.
        status (str): Indicates the outcome of the request; zero is success.
    """

    request_id: str
    status: str


class CheckCodeResponse(BaseModel):
    """Response object for checking a verification code.

    Args:
        request_id (str): The unique ID of the Verify request to check.
        status (str): Indicates the outcome of the request; zero is success.
        event_id (str): The ID of the verification event, such as an SMS or TTS call.
        price (str): The cost incurred for this request.
        currency (str): The currency code.
        estimated_price_messages_sent (str, Optional): Cost (in EUR) of the calls made
            and messages sent for the verification process.
    """

    request_id: str
    status: str
    event_id: str
    price: str
    currency: str
    estimated_price_messages_sent: Optional[str] = None


class Check(BaseModel):
    """The list of checks made for a specific verification and their outcomes.

    Args:
        date_received (str, Optional): The date and time this check was received (in the
            format YYYY-MM-DD HH:MM:SS)
        code (str, Optional): The code supplied with this check request.
        status (str, Optional): The status of the check.
        ip_address (str, Optional): The IP address of the check. This field is no longer
            used.
    """

    date_received: Optional[str] = None
    code: Optional[str] = None
    status: Optional[str] = None
    ip_address: Optional[str] = None


class Event(BaseModel):
    """The events that have taken place to verify this number, and their unique
    identifiers.

    Args:
        type (str, Optional): The type of event.
        id (str, Optional): The ID of the event.
    """

    type: Optional[str] = None
    id: Optional[str] = None


class VerifyStatus(BaseModel):
    """The status of a verification request.

    Args:
        request_id (str, Optional): The `request_id` that you received in the response to
            the Verify request and used in the Verify search request.
        account_id (str, Optional): The Vonage account ID the request was for.
        status (str, Optional): The status of the verification request.
        number (str, Optional): The phone number used in the request.
        price (str, Optional): The cost of this verification.
        currency (str, Optional): The currency code.
        sender_id (str, Optional): The sender ID provided in the Verify request.
        date_submitted (str, Optional): The date and time this verification request was
            submitted (in the format YYYY-MM-DD HH:MM:SS).
        date_finalized (str, Optional): The date and time this verification request was
            completed (in the format YYYY-MM-DD HH:MM:SS).
        first_event_date (str, Optional): The date and time of the first verification
            attempt (in the format YYYY-MM-DD HH:MM:SS).
        last_event_date (str, Optional): The date and time of the last verification
            attempt (in the format YYYY-MM-DD HH:MM:SS).
        checks (list[Check], Optional): The list of checks made for this verification and
            their outcomes.
        events (list[Event], Optional): The events that have taken place to verify this
            number, and their unique identifiers.
        estimated_price_messages_sent (str, Optional): Cost (in EUR) of the calls made
            and messages sent for the verification process.
    """

    request_id: Optional[str] = None
    account_id: Optional[str] = None
    status: Optional[str] = None
    number: Optional[str] = None
    price: Optional[str] = None
    currency: Optional[str] = None
    sender_id: Optional[str] = None
    date_submitted: Optional[str] = None
    date_finalized: Optional[str] = None
    first_event_date: Optional[str] = None
    last_event_date: Optional[str] = None
    checks: Optional[list[Check]] = None
    events: Optional[list[Event]] = None
    estimated_price_messages_sent: Optional[str] = None


class VerifyControlStatus(BaseModel):
    """The status of a verification control request.

    Args:
        status (str): The status of the control request.
        command (str): The command that was requested when cancelling a verify request
            or triggering the next workflow in a request.
    """

    status: str
    command: str


class NetworkUnblockStatus(BaseModel):
    """The status of a network unblock request.

    Args:
        network (str): The unique network ID of the network that was unblocked.
        unblocked_until (str): The date and time until which the network is unblocked.
    """

    network: str
    unblocked_until: str



from logging import getLogger
from typing import Literal, Optional

from pydantic import BaseModel, Field, model_validator
from vonage_utils.types import PhoneNumber

from .language_codes import LanguageCode, Psd2LanguageCode

logger = getLogger('vonage_verify')


class BaseVerifyRequest(BaseModel):
    """Base request object containing the data and options for a verification request.

    Args:
        number (PhoneNumber): The phone number to verify. Unless you are setting country
            explicitly, this number must be in E.164 format.
        country (str, Optional): If you do not provide `number` in international format
            or you are not sure if `number` is correctly formatted, specify the
            two-character country code in country. Verify will then format the number for
            you.
        code_length (int, Optional): The length of the verification code to generate.
        pin_expiry (int, Optional): How long the generated verification code is valid
            for, in seconds. When you specify both `pin_expiry` and `next_event_wait`
            then `pin_expiry` must be an integer multiple of `next_event_wait` otherwise
            `pin_expiry` is defaulted to equal `next_event_wait`.
        next_event_wait (int, Optional): The wait time in seconds between attempts to
            deliver the verification code.
        workflow_id (int, Optional): Selects the predefined sequence of SMS and TTS (Text
            To Speech) actions to use in order to convey the PIN to your user.
    """

    number: PhoneNumber
    country: Optional[str] = Field(None, max_length=2)
    code_length: Optional[Literal[4, 6]] = None
    pin_expiry: Optional[int] = Field(None, ge=60, le=3600)
    next_event_wait: Optional[int] = Field(None, ge=60, le=900)
    workflow_id: Optional[int] = Field(None, ge=1, le=7)

    @model_validator(mode='after')
    def check_expiry_and_next_event_timing(self):
        if self.pin_expiry is None or self.next_event_wait is None:
            return self
        if self.pin_expiry % self.next_event_wait != 0:
            logger.warning(
                f'The pin_expiry should be a multiple of next_event_wait.'
                f'\nThe current values are: pin_expiry={self.pin_expiry}, next_event_wait={self.next_event_wait}.'
                f'\nThe value of pin_expiry will be set to next_event_wait.'
            )
            self.pin_expiry = self.next_event_wait
        return self


class VerifyRequest(BaseVerifyRequest):
    """Request object for a verification request.

    You must set the `number` and `brand` fields.

    Args:
        number (PhoneNumber): The phone number to verify. Unless you are setting country
            explicitly, this number must be in E.164 format.
        country (str, Optional): If you do not provide `number` in international format
            or you are not sure if `number` is correctly formatted, specify the
            two-character country code in country. Verify will then format the number for
            you.
        brand (str): The name of the company or service that is sending the verification
            request. This will appear in the body of the SMS or TTS message.
        code_length (int, Optional): The length of the verification code to generate.
        pin_expiry (int, Optional): How long the generated verification code is valid
            for, in seconds. When you specify both `pin_expiry` and `next_event_wait`
            then `pin_expiry` must be an integer multiple of `next_event_wait` otherwise
            `pin_expiry` is defaulted to equal `next_event_wait`.
        next_event_wait (int, Optional): The wait time in seconds between attempts to
            deliver the verification code.
        workflow_id (int, Optional): Selects the predefined sequence of SMS and TTS (Text
            To Speech) actions to use in order to convey the PIN to your user.
        sender_id (str, Optional): An 11-character alphanumeric string that represents the
            sender of the verification request. Depending on the location of the phone
            number, restrictions may apply.
        lg (LanguageCode, Optional): The language to use for the verification message.
        pin_code (str, Optional): The verification code to send to the user. If you do not
            provide this, Vonage will generate a code for you.
    """

    brand: str = Field(..., max_length=18)
    sender_id: Optional[str] = Field(None, max_length=11)
    lg: Optional[LanguageCode] = None
    pin_code: Optional[str] = Field(None, min_length=4, max_length=10)


class Psd2Request(BaseVerifyRequest):
    """Request object for a PSD2 verification request.

    You must set the `number`, `payee` and `amount` fields.

    Args:
        number (PhoneNumber): The phone number to verify. Unless you are setting country
            explicitly, this number must be in E.164 format.
        payee (str): An alphanumeric string to indicate to the user the name of the
            recipient that they are confirming a payment to.
        amount (float): The decimal amount of the payment to be confirmed, in Euros.
        country (str, Optional): If you do not provide `number` in international
            format or you are not sure if `number` is correctly formatted, specify the
            two-character country code in `country`. Verify will then format the number for
            you.
        lg (Psd2LanguageCode, Optional): The language to use for the verification message.
        code_length (int, Optional): The length of the verification code to generate.
        pin_expiry (int, Optional): How long the generated verification code is valid
            for, in seconds. When you specify both `pin_expiry` and `next_event_wait`
            then `pin_expiry` must be an integer multiple of `next_event_wait` otherwise
            `pin_expiry` is defaulted to equal `next_event_wait`.
        next_event_wait (int, Optional): The wait time in seconds between attempts to
            deliver the verification code.
        workflow_id (int, Optional): Selects the predefined sequence of SMS and TTS (Text
            To Speech) actions to use in order to convey the PIN to your user.
    """

    payee: str = Field(..., max_length=18)
    amount: float
    lg: Optional[Psd2LanguageCode] = None
